#ifndef __BT_EXT_PORTING_H_
#define __BT_EXT_PORTING_H_
#include "uni_log.h"
#include "tuya_cloud_com_defs.h"
#include "tuya_cloud_types.h"
#include "tuya_hal_thread.h"
#include "tuya_uart.h"
#include "tuya_os_adapter.h"
#include "uni_queue.h"
#include "uni_thread.h"


// BTUART模块标识日志
#define TUYA_BT_UART_MODE           "[TUYA-BTUART]"
#ifdef TUYA_BT_UART_MODE
#define TUYA_BTUART_DEBUG(M, ...)  PR_DEBUG("%s" M,TUYA_BT_UART_MODE, ##__VA_ARGS__)
#define TUYA_BTUART_NOTICE(M, ...) PR_NOTICE("%s" M, TUYA_BT_UART_MODE, ##__VA_ARGS__)
#define TUYA_BTUART_ERR(M, ...)    PR_ERR("%s" M, TUYA_BT_UART_MODE, ##__VA_ARGS__)
#define TUYA_BTUART_RAW(M, ...)    PR_DEBUG_RAW("%s" M, TUYA_BT_UART_MODE, ##__VA_ARGS__)
#define TUYA_BTUART_INFO(M, ...)   PR_INFO("%s" M, TUYA_BT_UART_MODE, ##__VA_ARGS__)
#else
#define TUYA_BTUART_DEBUG  PR_DEBUG
#define TUYA_BTUART_NOTICE PR_NOTICE
#define TUYA_BTUART_ERR    PR_ERR
#define TUYA_BTUART_RAW    PR_DEBUG_RAW
#define TUYA_BTUART_INFO   PR_INFO
#endif
#define TUYA_GPS_KERNAL_HEAP_SIZE   2048
#define TUYA_GPS_KERNAL_PRIORITY    TRD_PRIO_1

#define PROT_VER                    0x00
#define MAX_RECV_COUNT              5

#define HN_RECV_BUF_MAX             256
#define HN_RECV_MIN_NUM             7       // 帧数据最小长度

#define TUYA_BTUART_RESEND_CNT_MAX  3       // 重发次数

#define EXT_BLE_UART_WAKEUP         0

#define BT_MAC_ADDR_PARAM           "bt_mac_addr_param"

typedef struct
{
    UCHAR_T *proc_buf;       // 串口接收缓存
    UINT_T buff_len;          // 串口接收缓存长度
    UINT_T rcount;            // 串口接收
    THRD_HANDLE thread;
    QUEUE_HANDLE queue;
    BOOL_T is_enable_recv;
    UINT_T try_cnt;
    BOOL_T recv_thrad_running;
    UCHAR_T timer_out_cnt;
    UCHAR_T recv_state;
    UCHAR_T send_state;
} TY_EX_BT_MSG_S;

#pragma pack(1)

typedef struct
{
    WORD_T len;       //数据长度
    UCHAR_T data[0];  //数据内容
}BTUART_SEND_DATA_S;

typedef struct
{
    WORD_T  head;    //固定为0x55aa(大端格式)
    UCHAR_T version; //版本，升级扩展用
    UCHAR_T fr_type; //具体帧类型(命令字)
    WORD_T  len;     //数据长度(大端格式)
    UCHAR_T data[0]; //数据内容
}BTUART_FRAME_S;

#pragma pack()

typedef struct
{
    UCHAR_T fr_type; //具体帧类型(命令字)
    UCHAR_T data[HN_RECV_BUF_MAX]; //数据内容
}RECV_FRAME_S;

//mcu to ble
typedef enum
{
    CAT1_TO_BLE_CMD_IDLE                    =   0x00,
    CAT1_TO_BLE_CMD_SW_VERSION              =   0x01,  //Cat.1->BLE 软件版本号 0：读/关/取消 1：写/开/使能 下同
    CAT1_TO_BLE_CMD_HW_VERSION              =   0x02,  //Cat.1->BLE 获取硬件版本号
    CAT1_TO_BLE_CMD_DEVICE_STATE            =   0x03,  //Cat.1->BLE 状态查
    CAT1_TO_BLE_CMD_BLE_TX_POWER            =   0x04,  //Cat.1->BLE ble发射广播信功率 默认 0db
    CAT1_TO_BLE_CMD_BLE_MAC                 =   0x05,  //Cat.1->BLE 设置ble MAC
    CAT1_TO_BLE_CMD_ADV                     =   0x06,  //Cat.1->BLE 设置广播包
    CAT1_TO_BLE_CMD_SCANRSP                 =   0x07,  //Cat.1->BLE 设置扫描应答包
    CAT1_TO_BLE_CMD_ADV_INTERVAL            =   0x08,  //Cat.1->BLE 设置广播间隔范围
    CAT1_TO_BLE_CMD_CONNECT_INTERVAL        =   0x09,  //Cat.1->BLE 设置连接间隔范围
    CAT1_TO_BLE_CMD_BLE_ADV                 =   0x0a,  //Cat.1->BLE 控制蓝牙广播,开关广播
    CAT1_TO_BLE_CMD_SCAN_NAME_RSSI          =   0x0b,  //Cat.1->BLE 蓝牙模组扫描指定广播名的rssi
    CAT1_TO_BLE_CMD_SEND_DATA               =   0x0c,  //Cat.1->BLE 模块发送蓝牙数据
    CAT1_TO_BLE_CMD_DISCONNECT              =   0x0d,  //Cat.1->BLE 模块主动断开连接
    CAT1_TO_BLE_CMD_BAT_STATE               =   0x0e,  //Cat.1->BLE 小电池状态获取 0x30 （数据部分为 bat_t）---预留  48v电池检测改到io检测里
    CAT1_TO_BLE_CMD_HID_RSSI                =   0x0f,  //Cat.1->BLE 连接状态 app rssi
    CAT1_TO_BLE_CMD_FD50_03                 =   0x10,  //Cat.1->BLE 设置fd50下的03 用于涂鸦配对 
    CAT1_TO_BLE_CMD_BONDING_CTR             =   0x11,  //cat.1->BLE 控制HID配对功能
    CAT1_TO_BLE_CMD_GPIO                    =   0x12,  //Cat.1发送控制命令
    CAT1_TO_BLE_CMD_WAKEUP_CAT1_EN          =   0x13,  //cat1 设置ble主动发给cta1数据前拉脉冲唤醒cat1
    CAT1_TO_BLE_CMD_MAX

}CAT1_TO_BLE_CMD_E;

//ble to mcu
typedef enum
{
    BLE_TO_CAT1_CMD_IDLE                    =   0x50,
    BLE_TO_CAT1_CMD_DEVICE_STATE            =   0x51,  //BLE主动更新状态
    BLE_TO_CAT1_CMD_SEND_DATA               =   0x52,  //BLE发送透传数据
    BLE_TO_CAT1_CMD_GPIO                    =   0x53,  //gpio状态改变自动更新
    BLE_TO_CAT1_CMD_SLAVE_ENCRYPT           =   0x54,  //配对加密信息，（存储校验，16byte）
    BLE_TO_CAT1_CMD_MAX

}BLE_TO_CAT1_CMD_E;

typedef enum{
    BT_INITILAZIONG             = 0,
    BT_READY,
    BT_OPENED,
    BT_CLOSE,
    BT_CONNECTED,
    BT_PAIRED,
    BT_UNCONNECTED,
}BT_DEVIVCE_STATUS_E;

typedef enum {
    BT_IO_R = 0,
    BT_IO_W
}BT_IOCTL_TYPE_E;

typedef VOID (*TUYA_EXT_BT_GPIO_IQR_CB)(UCHAR_T gpio, INT_T value);
typedef VOID (*TUYA_QUERY_HID_RSSI_CB)(VOID);

OPERATE_RET tuya_ext_bt_gpio_irq_reg(TUYA_EXT_BT_GPIO_IQR_CB irq_callback);

OPERATE_RET tuya_query_hid_rssi_reg(TUYA_QUERY_HID_RSSI_CB query_callback);

OPERATE_RET tuya_ext_bt_ctrl_gpio(UCHAR_T gpio, BT_IOCTL_TYPE_E wr, INT_T *value);

OPERATE_RET tuya_ext_bt_uart_init(tuya_uart_port_t port,INT_T baudrate);

OPERATE_RET tuya_ext_query_status(BT_DEVIVCE_STATUS_E *stat);

OPERATE_RET tuya_ext_bt_init_mac(VOID);

OPERATE_RET tuya_ext_bt_set_mac(CONST NW_MAC_S *mac);

OPERATE_RET tuya_hid_bond_ctrl(BOOL_T isBond);

BOOL_T tuya_hid_is_bond(VOID);

OPERATE_RET tuya_bt_wakeup_enable(BOOL_T is_enable);

OPERATE_RET tuya_query_ble_hid_rssi(signed char *rssi);

#endif